unit TupleU;

{$mode ObjFPC}{$H+}

interface

uses
  Classes, SysUtils, fgl;

type

  TSpannungstensorArray = array of Single;

  { TNodeList }

  TNodeList = class(specialize TFPGMap<Integer{KnotenID}, TSpannungstensorArray>)
  public
    constructor Create;
  end;

  { TTupleList }

  TTupleList = class(specialize TFPGMapObject<Integer{ElementID}, TNodeList>)
  private
    function AddOrReplace(aElementID, aKnotenID: Integer; aSpannungstensorArray: TSpannungstensorArray; aReplace: Boolean): TPoint; //return ElementID index (X) and KnotenID index (Y)
  public
    constructor Create;
    //always add new values
    function Add(aElementID, aKnotenID: Integer; aSpannungstensorArray: TSpannungstensorArray): TPoint; //return ElementID index (X) and KnotenID index (Y)
    function Add(aElementID, aKnotenID: Integer; aSpannungstensors: array of Single): TPoint;
    function Add(aElementID, aKnotenID: Integer; aSpannungstensor: Single): TPoint;
    //if exists pair ElementID, KnotenID then replace array
    function Replace(aElementID, aKnotenID: Integer; aSpannungstensorArray: TSpannungstensorArray): TPoint; //return ElementID index (X) and KnotenID index (Y)
    function Get(aElementID, aKnotenID: Integer): TSpannungstensorArray;
    function IsExists(aElementID, aKnotenID: Integer): Boolean;
    procedure Delete(aElementID, aKnotenID: Integer);
    procedure Delete(aElementID: Integer);
  end;


implementation

{ TNodeList }

constructor TNodeList.Create;
begin
  inherited Create;
  Duplicates := dupIgnore;
  Sorted := True;
end;

{ TTupleList }

function TTupleList.AddOrReplace(aElementID, aKnotenID: Integer; aSpannungstensorArray: TSpannungstensorArray; aReplace: Boolean): TPoint;
var
  idxe, idxk, i, prevLen: Integer;
  nl: TNodeList;
  sarr: TSpannungstensorArray;
begin
  idxe := IndexOf(aElementID);
  if idxe < 0 then
  begin
    nl := TNodeList.Create;
    idxe := inherited Add(aElementID, nl);
  end;
  Result.X := idxe;
  idxk := Data[idxe].IndexOf(aKnotenID);
  if idxk >= 0 then
  begin
    Result.Y := idxk;
    if aReplace then
      Data[idxe].Data[idxk] := aSpannungstensorArray
    else
    begin
      sarr := Data[idxe].Data[idxk];
      prevLen := Length(sarr);
      SetLength(sarr, prevLen + Length(aSpannungstensorArray));
      for i := 0 to High(aSpannungstensorArray) do
        sarr[prevLen + i] := aSpannungstensorArray[i];
      Data[idxe].Data[idxk] := sarr;
    end;
  end
  else
    Result.Y := Data[idxe].Add(aKnotenID, aSpannungstensorArray);
end;

constructor TTupleList.Create;
begin
  inherited Create;
  Duplicates := dupIgnore;
  Sorted := True;
end;

function TTupleList.Add(aElementID, aKnotenID: Integer; aSpannungstensorArray: TSpannungstensorArray): TPoint;
begin
  Result := AddOrReplace(aElementID, aKnotenID, aSpannungstensorArray, False);
end;

function TTupleList.Add(aElementID, aKnotenID: Integer; aSpannungstensors: array of Single): TPoint;
var
  i: Integer;
  sarr: TSpannungstensorArray;
begin
  SetLength(sarr, Length(aSpannungstensors));
  for i := 0 to High(aSpannungstensors) do
    sarr[i] := aSpannungstensors[i];
  Result := AddOrReplace(aElementID, aKnotenID, sarr, False);
end;

function TTupleList.Add(aElementID, aKnotenID: Integer; aSpannungstensor: Single): TPoint;
var
  sarr: TSpannungstensorArray;
begin
  SetLength(sarr, 1);
  sarr[0] := aSpannungstensor;
  Result := AddOrReplace(aElementID, aKnotenID, sarr, False);
end;

function TTupleList.Replace(aElementID, aKnotenID: Integer; aSpannungstensorArray: TSpannungstensorArray): TPoint;
begin
  Result := AddOrReplace(aElementID, aKnotenID, aSpannungstensorArray, True);
end;

function TTupleList.Get(aElementID, aKnotenID: Integer): TSpannungstensorArray;
var
  idxe, idxk: Integer;
begin
  Result := nil;
  idxe := IndexOf(aElementID);
  if idxe >= 0 then
  begin
    idxk := Data[idxe].IndexOf(aKnotenID);
    if idxk >= 0 then
      Result := Data[idxe].Data[idxk];
  end;
end;

function TTupleList.IsExists(aElementID, aKnotenID: Integer): Boolean;
var
  idxe, idxk: Integer;
begin
  Result := False;
  idxe := IndexOf(aElementID);
  if idxe >= 0 then
  begin
    idxk := Data[idxe].IndexOf(aKnotenID);
    if idxk >= 0 then
      Result := True;
  end;
end;

procedure TTupleList.Delete(aElementID, aKnotenID: Integer);
var
  idxe, idxk: Integer;
begin
  idxe := IndexOf(aElementID);
  if idxe >= 0 then
  begin
    idxk := Data[idxe].IndexOf(aKnotenID);
    if idxk >= 0 then
      Data[idxe].Delete(idxk);
    if Data[idxe].Count = 0 then
      inherited Delete(idxe);
  end;
end;

procedure TTupleList.Delete(aElementID: Integer);
var
  idxe: Integer;
begin
  idxe := IndexOf(aElementID);
  if idxe >= 0 then
    inherited Delete(idxe);
end;

end.

